@@ -1,5 +1,6 @@
 /*   GTimeTracker - a time tracker
  *   Copyright (C) 1997,98 Eckehard Berns
+ *   Copyright (C) 2001 Linas Vepstas <linas@linas.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,22 +21,38 @@
 #include <gnome.h>
 #include <string.h>
 
+#include "app.h"
+#include "ctree.h"
+#include "cur-proj.h"
+#include "dialog.h"
+#include "err-throw.h"
+#include "file-io.h"
 #include "gtt.h"
+#include "menucmd.h"
+#include "menus.h"
+#include "prefs.h"
+#include "proj.h"
+#include "props-proj.h"
+#include "timer.h"
+#include "xml-gtt.h"
 
-/* XXX: this is our main window, perhaps it is a bit ugly this way and
- * should be passed around in the data fields */
-extern GtkWidget *window;
 
 void
 quit_app(GtkWidget *w, gpointer data)
 {
-	if (!project_list_save(NULL)) {
+	const char * errmsg;
+
+	errmsg = save_all ();
+	if (errmsg)
+	{
 		msgbox_ok(_("Warning"),
-			  _("I could not write the configuration file!"),
-			  GNOME_STOCK_BUTTON_OK,
-			  GTK_SIGNAL_FUNC(gtk_main_quit));
+		     errmsg,
+		     GNOME_STOCK_BUTTON_OK,
+		     GTK_SIGNAL_FUNC(gtk_main_quit));
+		g_free ((gchar *) errmsg);
 		return;
 	}
+
 	gtk_main_quit();
 }
 
@@ -44,31 +61,38 @@ quit_app(GtkWidget *w, gpointer data)
 void
 about_box(GtkWidget *w, gpointer data)
 {
-        static GtkWidget *about = NULL;
-        gchar *authors[] = {
-                "Eckehard Berns\n<eb@berns.i-s-o.net>",
-                NULL
-        };
+	 static GtkWidget *about = NULL;
+	 gchar *authors[] = {
+		  "Eckehard Berns\n<eb@berns.i-s-o.net>",
+		  "George Lebl\n<jirka@5z.com>",
+		  "Linas Vepstas\n<linas@linas.org>",
+		  "and dozens of bug-fixers and translators",
+		  NULL
+	 };
 	if (about != NULL)
 	{
 		gdk_window_show(about->window);
 		gdk_window_raise(about->window);
 		return;
 	}
-        about = gnome_about_new(APP_NAME,
-                                VERSION,
-                                "Copyright (C) 1997,98 Eckehard Berns",
-                                (const gchar **)authors,
+	about = gnome_about_new(APP_NAME,
+				    VERSION,
+				    "Copyright (C) 1997,98 Eckehard Berns and others",
+				    (const gchar **)authors,
 #ifdef DEBUG
-                                __DATE__ ", " __TIME__,
+				    __DATE__ ", " __TIME__,
 #else
-				_("Time tracking tool for GNOME"),
+ _("GTimeTracker is a combination stop-watch, diary, consultant billing "
+   "system and project manager.  You can measure the amount of time you "
+   "spend on a task, associate a memo with it, set a billing rate, print "
+   "an invoice, as well as track that status of other projects."),
+
 #endif
-                                NULL);
+				    NULL);
 	gtk_signal_connect(GTK_OBJECT(about), "destroy",
-			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
+		GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
 	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(window));
-        gtk_widget_show(about);
+	gtk_widget_show(about);
 }
 
 
@@ -78,14 +102,21 @@ static void
 project_name_desc(GtkWidget *w, GtkEntry **entries)
 {
 	char *name, *desc;
-	project *proj;
+	GttProject *proj;
+	GttProject *sib_prj;
+	
+	sib_prj = ctree_get_focus_project (global_ptw);
 
 	if (!(name = gtk_entry_get_text(entries[0]))) return;
-        if (!(desc = gtk_entry_get_text(entries[1]))) return;
+	if (!(desc = gtk_entry_get_text(entries[1]))) return;
 	if (!name[0]) return;
 
-	project_list_add(proj = project_new_title_desc(name, desc));
-        clist_add(proj);
+	/* New project will have the same parent as the currently
+	 * running project.  This seems like the sanest choice.
+	 */
+	proj = gtt_project_new_title_desc(name, desc);
+	gtt_project_insert_after (proj, sib_prj);
+	ctree_insert_after (global_ptw, proj, sib_prj);
 }
 
 static void
@@ -100,39 +131,39 @@ new_project(GtkWidget *widget, gpointer data)
 {
 	GtkWidget *dlg, *t, *title, *d, *desc;
 	GtkBox *vbox;
-        GtkWidget **entries = g_new0(GtkWidget *, 2);
-        GtkWidget *table;
+	GtkWidget **entries = g_new0(GtkWidget *, 2);
+	GtkWidget *table;
 
 	title = gnome_entry_new("project_title");
-        desc = gnome_entry_new("project_description");
-        entries[0] = gnome_entry_gtk_entry(GNOME_ENTRY(title));
-        entries[1] = gnome_entry_gtk_entry(GNOME_ENTRY(desc));
+	desc = gnome_entry_new("project_description");
+	entries[0] = gnome_entry_gtk_entry(GNOME_ENTRY(title));
+	entries[1] = gnome_entry_gtk_entry(GNOME_ENTRY(desc));
 
 	new_dialog_ok_cancel(_("New Project..."), &dlg, &vbox,
 			     GNOME_STOCK_BUTTON_OK,
 			     GTK_SIGNAL_FUNC(project_name_desc),
-                             entries,
+				 entries,
 			     GNOME_STOCK_BUTTON_CANCEL, NULL, NULL);
 
 	t = gtk_label_new(_("Project Title"));
-        d = gtk_label_new(_("Description"));
+	d = gtk_label_new(_("Description"));
 
-        table = gtk_table_new(2,2, FALSE);
-        gtk_table_attach(GTK_TABLE(table), t,     0,1, 0,1,
-                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
-        gtk_table_attach(GTK_TABLE(table), title, 1,2, 0,1,
-                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
-        gtk_table_attach(GTK_TABLE(table), d,     0,1, 1,2,
-                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
-        gtk_table_attach(GTK_TABLE(table), desc,  1,2, 1,2,
-                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
+	table = gtk_table_new(2,2, FALSE);
+	gtk_table_attach(GTK_TABLE(table), t,     0,1, 0,1,
+			    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
+	gtk_table_attach(GTK_TABLE(table), title, 1,2, 0,1,
+			    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
+	gtk_table_attach(GTK_TABLE(table), d,     0,1, 1,2,
+			    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
+	gtk_table_attach(GTK_TABLE(table), desc,  1,2, 1,2,
+			    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 2, 1);
 
-        gtk_box_pack_start(vbox, table, FALSE, FALSE, 2);
+	gtk_box_pack_start(vbox, table, FALSE, FALSE, 2);
 	gtk_widget_show(t);
 	gtk_widget_show(title);
-        gtk_widget_show(d);
-        gtk_widget_show(desc);
-        gtk_widget_show(table);
+	gtk_widget_show(d);
+	gtk_widget_show(desc);
+	gtk_widget_show(table);
 
 	gtk_widget_grab_focus(entries[0]);
 
@@ -155,19 +186,33 @@ new_project(GtkWidget *widget, gpointer data)
 static void
 init_project_list_2(GtkWidget *widget, int button)
 {
-	GtkWidget *dlg, *t;
-	GtkBox *vbox;
-	
+	GttErrCode conf_errcode;
+
 	if (button != 0) return;
-	if (!project_list_load(NULL)) {
-		new_dialog_ok(_("Warning"), &dlg, &vbox,
-			      GNOME_STOCK_BUTTON_OK, NULL, NULL);
-		t = gtk_label_new(_("I could not read the configuration file"));
-		gtk_widget_show(t);
-		gtk_box_pack_start(vbox, t, TRUE, FALSE, 2);
-		gtk_widget_show(dlg);
-	} else {
-                setup_clist();
+
+	/* Try ... */
+	gtt_err_set_code (GTT_NO_ERR);
+	gtt_load_config (NULL);
+
+	/* Catch ... */
+	conf_errcode = gtt_err_get_code();
+	if (GTT_NO_ERR != conf_errcode)
+	{
+		const char *fp, *errmsg;
+		fp = gtt_get_config_filepath();
+		errmsg = gtt_err_to_string (conf_errcode, fp);
+		msgbox_ok(_("Warning"),
+			errmsg, GNOME_STOCK_BUTTON_OK, NULL);
+
+		g_free ((gchar *) errmsg);
+	}
+	else
+	{
+
+		gtt_post_data_config ();
+		ctree_setup(global_ptw);
+		menus_add_plugins (GNOME_APP(window));
+		app_show();
 	}
 }
 
@@ -177,8 +222,7 @@ void
 init_project_list(GtkWidget *widget, gpointer data)
 {
 	msgbox_ok_cancel(_("Reload Configuration File"),
-			 _("This will overwrite your current set of projects.\n"
-			   "Do you really want to reload the configuration file?"),
+			 _("Do you really want to reload the configuration file?"),
 			 GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
 			 GTK_SIGNAL_FUNC(init_project_list_2));
 }
@@ -188,17 +232,16 @@ init_project_list(GtkWidget *widget, gpointer data)
 void
 save_project_list(GtkWidget *widget, gpointer data)
 {
-	if (!project_list_save(NULL)) {
-		GtkWidget *dlg, *t;
-		GtkBox *vbox;
-		
-		new_dialog_ok(_("Warning"), &dlg, &vbox,
-			      GNOME_STOCK_BUTTON_OK, NULL, NULL);
-		t = gtk_label_new(_("I could not write the configuration file!"));
-		gtk_widget_show(t);
-		gtk_box_pack_start(vbox, t, FALSE, FALSE, 2);
-		gtk_widget_show(dlg);
-		return;
+	const char * errmsg;
+
+	errmsg = save_all ();
+	if (errmsg)
+	{
+		msgbox_ok(_("Warning"),
+		     errmsg,
+		     GNOME_STOCK_BUTTON_OK,
+		     NULL);
+		g_free ((gchar *) errmsg);
 	}
 }
 
@@ -266,19 +309,25 @@ export_current_state (GtkWidget *widget, gpointer data)
 }
 
 
-project *cutted_project = NULL;
+GttProject *cutted_project = NULL;
 
 void
 cut_project(GtkWidget *w, gpointer data)
 {
-	if (!cur_proj) return;
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
+
+	if (!prj) return;
 	if (cutted_project)
-		project_destroy(cutted_project);
-	cutted_project = cur_proj;
+	{
+		gtt_project_destroy(cutted_project);
+	}
+	cutted_project = prj;
 	prop_dialog_set_project(NULL);
-	project_list_remove(cur_proj);
-	cur_proj_set(NULL);
-        clist_remove(cutted_project);
+
+	gtt_project_remove(cutted_project);
+	if (cutted_project == cur_proj) cur_proj_set(NULL);
+	ctree_remove(global_ptw, cutted_project);
 }
 
 
@@ -286,19 +335,27 @@ cut_project(GtkWidget *w, gpointer data)
 void
 paste_project(GtkWidget *w, gpointer data)
 {
-	int pos;
-	project *p;
+	GttProject *sib_prj;
+	GttProject *p;
 	
+	sib_prj = ctree_get_focus_project (global_ptw);
+
 	if (!cutted_project) return;
-	p = project_dup(cutted_project);
-	if (!cur_proj) {
-                clist_add(p);
-		project_list_add(p);
+	p = cutted_project;
+
+	/* if we paste a second time, we better paste a copy ... */
+	cutted_project = gtt_project_dup(cutted_project);
+
+	/* insert before the focus proj */
+	gtt_project_insert_before (p, sib_prj);
+
+	if (!sib_prj) 
+	{
+		/* top-level insert */
+		ctree_add(global_ptw, p, NULL);
 		return;
 	}
-        pos = cur_proj->row;
-	project_list_insert(p, pos);
-        clist_insert(p, pos);
+	ctree_insert_before(global_ptw, p, sib_prj);
 }
 
 
@@ -306,10 +363,16 @@ paste_project(GtkWidget *w, gpointer data)
 void
 copy_project(GtkWidget *w, gpointer data)
 {
-	if (!cur_proj) return;
-	if (cutted_project)
-		project_destroy(cutted_project);
-	cutted_project = project_dup(cur_proj);
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
+
+	if (!prj) return;
+
+	if (cutted_project) 
+	{
+		gtt_project_destroy(cutted_project);
+	}
+	cutted_project = gtt_project_dup(prj);
 	menu_set_states(); /* to enable paste */
 }
 
@@ -323,8 +386,9 @@ copy_project(GtkWidget *w, gpointer data)
 void
 menu_start_timer(GtkWidget *w, gpointer data)
 {
-	start_timer();
-	menu_set_states();
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
+	cur_proj_set (prj);
 }
 
 
@@ -332,28 +396,29 @@ menu_start_timer(GtkWidget *w, gpointer data)
 void
 menu_stop_timer(GtkWidget *w, gpointer data)
 {
-	stop_timer();
-	menu_set_states();
+	cur_proj_set (NULL);
 }
 
 
 void
 menu_toggle_timer(GtkWidget *w, gpointer data)
 {
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
+
 	/* if (GTK_CHECK_MENU_ITEM(menus_get_toggle_timer())->active) { */
-	if (main_timer == 0) {
-		start_timer();
+	if (timer_is_running()) {
+		cur_proj_set (NULL);
 	} else {
-		stop_timer();
+		cur_proj_set (prj);
 	}
-	menu_set_states();
 }
 
 
 void
 menu_options(GtkWidget *w, gpointer data)
 {
-	options_dialog();
+	prefs_dialog_show();
 }
 
 
@@ -361,8 +426,11 @@ menu_options(GtkWidget *w, gpointer data)
 void
 menu_properties(GtkWidget *w, gpointer data)
 {
-	if (cur_proj) {
-		prop_dialog(cur_proj);
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
+
+	if (prj) {
+		prop_dialog_show(prj);
 	}
 }
 
@@ -371,9 +439,11 @@ menu_properties(GtkWidget *w, gpointer data)
 void
 menu_clear_daily_counter(GtkWidget *w, gpointer data)
 {
-	g_return_if_fail(cur_proj != NULL);
+	GttProject *prj;
+	prj = ctree_get_focus_project (global_ptw);
 
-	cur_proj->day_secs = 0;
-        clist_update_label(cur_proj);
+	gtt_clear_daily_counter (prj);
+	ctree_update_label(global_ptw, prj);
 }
 
+/* ============================ END OF FILE ======================= */
